﻿/*! 
	\file tstdata.h
	\brief opc 测试历程数据模拟  
 
	opc 测试历程数据模拟
  
	\author 重庆唐码软件有限公司
	*/
#ifndef TSTDATA_H
#define TSTDATA_H
#pragma warning(disable : 4996)
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "opcsrv_da/opcsrv_da.h"

class CTstData
{
public:
	CTstData(){
	}
	~CTstData(){
	}
public:
	
	static bool __stdcall SetItemVal(const char* sTagName,VARIANT* pval,unsigned short wqa,long long filetime,void* pCallBackArg) // 写标签值
	{
		//这里只是简单的写到opcserver,实际可根据情况写到现场设备
		return !opcsrv_setval(sTagName,pval,wqa,filetime);
	}
	
	void AddTag()//添加测试标签
	{
		//只读标签
		opcsrv_addtag("r_i32","des_ri32","unit_ri32",ODT_I32,OTEM_NOWRITE);
		opcsrv_addtag("r_i64","des_ri64","unit_ri64",ODT_I64,OTEM_NOWRITE);
		opcsrv_addtag("r_f32","des_rf32","unit_rf32",ODT_F32,OTEM_NOWRITE);
		opcsrv_addtag("r_f64","des_rf64","unit_rf64",ODT_F64,OTEM_NOWRITE);
		opcsrv_addtag("r_str","des_rstr","unit_rstr",ODT_STR,OTEM_NOWRITE);
		opcsrv_addtag("r_blob","des_rblob","unit_blob",ODT_BLOB,OTEM_NOWRITE);

		//可写标签
		opcsrv_addtag("w_i32","des_wi32","unit_wi32",ODT_I32,OTEM_WRITE);
		opcsrv_addtag("w_i64","des_wi64","unit_wi64",ODT_I64,OTEM_WRITE);
		opcsrv_addtag("w_f32","des_wf32","unit_wf32",ODT_F32,OTEM_WRITE);
		opcsrv_addtag("w_f64","des_wf64","unit_wf64",ODT_F64,OTEM_WRITE);
		opcsrv_addtag("w_str","des_wstr","unit_wstr",ODT_STR,OTEM_WRITE);
		opcsrv_addtag("w_blob","des_wblob","unit_wblob",ODT_BLOB,OTEM_WRITE);
        
        //以下测试标签分级查询
        opcsrv_addtag("factory1.device1.r_i32", "des_ri32", "unit_ri32", ODT_I32, OTEM_NOWRITE);
        opcsrv_addtag("factory1.device1.r_f32", "des_rf32", "unit_rf32", ODT_F32, OTEM_NOWRITE);

        opcsrv_addtag("factory1.device2.r_i32", "des_ri32", "unit_ri32", ODT_I32, OTEM_NOWRITE);
        opcsrv_addtag("factory1.device2.r_f32", "des_rf32", "unit_rf32", ODT_F32, OTEM_NOWRITE);

        opcsrv_addtag("factory2.device1.r_i32", "des_ri32", "unit_ri32", ODT_I32, OTEM_NOWRITE);
        opcsrv_addtag("factory2.device1.r_f32", "des_rf32", "unit_rf32", ODT_F32, OTEM_NOWRITE);

        opcsrv_addtag("factory2.device2.r_i32", "des_ri32", "unit_ri32", ODT_I32, OTEM_NOWRITE);
        opcsrv_addtag("factory2.device2.r_f32", "des_rf32", "unit_rf32", ODT_F32, OTEM_NOWRITE);
	}

	//替代 opcsrv_setstr,功能完全相同
	bool SetStrVal(const char* tagname,const char* sval,unsigned short wqa,long long filetime)
	{
		VARIANT val;
		VariantInit(&val);
		size_t len = strlen(sval);
		wchar_t *pwstr = (wchar_t*)malloc(sizeof(wchar_t) * (len + 1));
		if(!pwstr)
			return false;		
		MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,sval, -1, pwstr, len+1);
		val.vt = VT_BSTR;
		val.bstrVal = ::SysAllocString(pwstr);
		int nret = opcsrv_setval(tagname,&val,OPC_QUALITY_GOOD,0);
		VariantClear(&val);
		free(pwstr);
		return !nret;
	}

	//替代 opcsrv_setblob,功能完全相同
	bool SetBlobVal(const char* tagname,unsigned char* pval,int nsize,unsigned short wqa,long long filetime)
	{
		VARIANT val;
		if(!pval || !nsize)
			return false;
		VariantInit(&val);
		val.vt = VT_UI1|VT_ARRAY;
		SAFEARRAYBOUND sabound[1];
		sabound[0].cElements = nsize;
		sabound[0].lLbound = 0;
			
		SAFEARRAY *psa = SafeArrayCreate(VT_UI1, 1, sabound);
		if (!psa) 
			return false;
		
		unsigned char *pc = nullptr;
		SafeArrayAccessData(psa, reinterpret_cast<void HUGEP**>(&pc));
		if(!pc)
		{
			SafeArrayDestroy(psa);
			return false;
		}
		memcpy(pc, pval,nsize); 
		SafeArrayUnaccessData(psa);
		val.parray = psa;
		
		int nret = opcsrv_setval(tagname,&val,wqa,filetime);
		VariantClear(&val);		
		return !nret;
	}
	void SimuVal(time_t t)//模拟值
	{
		VARIANT val;
		VariantInit(&val);

		val.vt = VT_I4;val.lVal = (int)(t%10000);
		opcsrv_setval("r_i32",&val,OPC_QUALITY_GOOD,0);
        opcsrv_setval("factory1.device1.r_i32",&val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory1.device2.r_i32", &val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory2.device1.r_i32", &val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory2.device2.r_i32", &val, OPC_QUALITY_GOOD, 0);

		val.vt = VT_R4;val.fltVal = (float)((t%10000)/100.0);
		opcsrv_setval("r_f32",&val,OPC_QUALITY_GOOD,0);
        opcsrv_setval("factory1.device1.r_f32", &val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory1.device2.r_f32", &val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory2.device1.r_f32", &val, OPC_QUALITY_GOOD, 0);
        opcsrv_setval("factory2.device2.r_f32", &val, OPC_QUALITY_GOOD, 0);

		val.vt = VT_I8;val.llVal = t;
		opcsrv_setval("r_i64",&val,OPC_QUALITY_GOOD,0);

		val.vt = VT_R8;val.dblVal = (t%1000000)/1000.0;
		opcsrv_setval("r_f64",&val,OPC_QUALITY_GOOD,0);

		VariantClear(&val);//简单类型,不调用VariantClear也行
		
		char sval[256]={0};
		struct tm *ptm = localtime(&t);
		if(ptm)
			sprintf(sval,"%04d-%02d-%02d %02d:%02d:%02d",ptm->tm_year + 1900,ptm->tm_mon + 1,ptm->tm_mday,ptm->tm_hour,ptm->tm_min,ptm->tm_sec);
	
//		opcsrv_setstr("r_str",sval,OPC_QUALITY_GOOD,0);
//		opcsrv_setblob("r_blob",(unsigned char*)sval,strlen(sval),OPC_QUALITY_GOOD,0);
		SetStrVal("r_str",sval,OPC_QUALITY_GOOD,0);
		SetBlobVal("r_blob",(unsigned char*)sval,strlen(sval),OPC_QUALITY_GOOD,0);
	}
};
#endif