﻿/*!
\file tstopc_atlsrv.cpp

\author jiangyong
\date 2021-9-5

*/
#define _WIN32_WINNT 0x0600 // windows server 2008 及以后版本,也可以定义工程配置里

#ifndef STRICT
#define STRICT
#endif
#define _ATL_FREE_THREADED
#define _ATL_NO_AUTOMATIC_NAMESPACE
#define _ATL_CSTRING_EXPLICIT_CONSTRUCTORS
#define ATL_NO_ASSERT_ON_DESTROY_NONEXISTENT_WINDOW

#include <atlbase.h>
#include <atlcom.h>
#include <atlctl.h>
#include <time.h>

#include "opcsrv_da/opcsrv_da.h"
#pragma comment(lib, "opcsrv_da/opcsrv_da.lib")

#define opc_service_name "tstopc_atlsrv" // OPC Server的 服务名

const char* g_sClsID = "{05A87ED4-4855-4106-A6A9-89C8B0890E98}";// CLSID 用GUID工具产生的Registry Format, 唯一的，自己的程序要更改这个值
const char* g_sVendor = "tom"; // 提供商,比如"tom"
const char* g_sDrivername = opc_service_name;// 驱动名,比如"rdb4opcda",如果是后台服务，注意这个名字要和服务名完全一样
const char* g_sVersion = "1";// 版本名,比如"1"
const char*	g_sOpcServerDes = opc_service_name;// OPC Server描述 ，<128字符,  如果是后台服务，同时也是服务名(切记)

#define OPCSERVICENAME _T(opc_service_name)

#include "tstdata.h" //测试数据类
class COpcSrv //包装一个线程,让OPC DA Server在一个线程中运行。
{
public:
	COpcSrv()
	{
		_lkill = 0;
		_lrun = 0;
		_hwait = ::CreateEvent(nullptr, false, false, nullptr);
	}
	~COpcSrv()
	{
		if (_hwait != nullptr)
			::CloseHandle(_hwait);
	}
private:
	long    _lkill, _lrun;
	HANDLE  _hwait;
	CTstData _tstdata;

	static void ThreadProcess(void* pargs)
	{
		COpcSrv* pt = (COpcSrv*)pargs;
		pt->DoRun();
	}
	void DoRun()
	{
		time_t told = 0, t;
		InterlockedExchange(&_lkill, 0);
		InterlockedExchange(&_lrun, 1);
		while (!InterlockedExchangeAdd(&_lkill, 0)) {
			WaitForSingleObject(_hwait, 100);
			t = ::time(nullptr);
			if (told != t) { //每秒模拟一次数据
				_tstdata.SimuVal(t);
				told = t;
			}
		}
		InterlockedExchange(&_lkill, 0);
		InterlockedExchange(&_lrun, 0);
	}
public:
	bool Start()
	{
		if (InterlockedExchangeAdd(&_lrun, 0))
			return true;
		_tstdata.AddTag();//添加测试标签
		int nret = opcsrv_start(g_sClsID, g_sVendor, g_sDrivername, g_sVersion, g_sOpcServerDes, _tstdata.SetItemVal, nullptr, true);//启动，锁定OPC服务
		if (nret != OE_OK)
			return false;
		_beginthread(ThreadProcess, 0, this);
		return true;
	}
	void Stop()
	{
		InterlockedExchange(&_lkill, 1);
		while (InterlockedExchangeAdd(&_lrun, 0))
			Sleep(100);
	}
};

using namespace ATL;

#include <stdio.h>

class CatlopcsrvModule : public ATL::CAtlServiceModuleT< CatlopcsrvModule, 0 >
{
protected:
	COpcSrv* _pclsopc;
public:
	CatlopcsrvModule() : _pclsopc(nullptr) {
		_tcscpy(m_szServiceName, OPCSERVICENAME);
	}

	HRESULT InitializeSecurity() throw()
	{
		return S_OK;
	}

	HRESULT Run(_In_ int nShowCmd = SW_HIDE) throw()//重载基类的运行启动OPC
	{
		HRESULT hr = S_OK;
		_pclsopc = new COpcSrv();
		if (_pclsopc) {
			if (!_pclsopc->Start()) {
				_pclsopc->Stop();
				delete _pclsopc;
				_pclsopc = nullptr;
				return E_FAIL;
			}
		}
		SetServiceStatus(SERVICE_RUNNING); //报告已经成功启动

		RunMessageLoop(); //windows消息循环

		hr = PostMessageLoop();
		if (_pclsopc) {
			_pclsopc->Stop();
			delete _pclsopc;
			_pclsopc = nullptr;
		}
		return hr;
	}

	BOOL Install_() //自己安装服务,不要用ATL提供的。
	{
		if (IsInstalled())
			return TRUE;

		SC_HANDLE hSCM = ::OpenSCManager(nullptr, nullptr, SC_MANAGER_ALL_ACCESS);
		if (!hSCM) {
			MessageBox(nullptr, _T("Couldn't open service manager"), m_szServiceName, MB_OK);
			return FALSE;
		}

		TCHAR szFilePath[_MAX_PATH];
		::GetModuleFileName(nullptr, szFilePath, _MAX_PATH);

		TCHAR szBinfile[_MAX_PATH];
		_tcscpy(szBinfile, szFilePath);
		_tcscat(szBinfile, _T(" -service"));

		SC_HANDLE hService = ::CreateService(
			hSCM, m_szServiceName, m_szServiceName,
			0, SERVICE_WIN32_OWN_PROCESS,
			SERVICE_AUTO_START,
			SERVICE_ERROR_IGNORE,
			szBinfile, nullptr, nullptr, nullptr, nullptr, nullptr);

		if (!hService) {
			::CloseServiceHandle(hSCM);
			MessageBox(nullptr, _T("Couldn't create service"), m_szServiceName, MB_OK);
			return FALSE;
		}

		TCHAR sKey[_MAX_PATH];
		_tcscpy(sKey, _T("SYSTEM\\CurrentControlSet\\Services\\"));
		_tcscat(sKey, m_szServiceName);

		HKEY hkey;
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKey, 0, KEY_WRITE | KEY_READ, &hkey) == ERROR_SUCCESS) {
			RegSetValueEx(hkey, _T("Description"), 0, REG_SZ, (LPBYTE)(m_szServiceName), (lstrlen(m_szServiceName) + 1) * sizeof(TCHAR));
			RegCloseKey(hkey);
		}

		::CloseServiceHandle(hService);
		::CloseServiceHandle(hSCM);

		MessageBox(nullptr, _T("install service success!"), m_szServiceName, MB_OK);
		return TRUE;
	}

	BOOL Uninstall_() //自己卸载服务,不要用ATL提供的。
	{
		if (!IsInstalled())
			return TRUE;

		SC_HANDLE hSCM = ::OpenSCManager(nullptr, nullptr, SC_MANAGER_ALL_ACCESS);
		if (!hSCM) {
			MessageBox(nullptr, _T("Couldn't open service manager"), m_szServiceName, MB_OK);
			return FALSE;
		}

		SC_HANDLE hService = ::OpenService(hSCM, m_szServiceName, SERVICE_STOP | DELETE);
		if (!hService) {
			::CloseServiceHandle(hSCM);
			MessageBox(nullptr, _T("Couldn't open service"), m_szServiceName, MB_OK);
			return FALSE;
		}
		SERVICE_STATUS status;
		::ControlService(hService, SERVICE_CONTROL_STOP, &status);

		BOOL bDelete = ::DeleteService(hService);
		::CloseServiceHandle(hService);
		::CloseServiceHandle(hSCM);

		if (bDelete) {
			MessageBox(nullptr, _T("delete service success!"), m_szServiceName, MB_OK);
			return TRUE;
		}

		MessageBox(nullptr, _T("Service could not be deleted"), m_szServiceName, MB_OK);
		return FALSE;
	}

	BOOL RegisterServer_() //注册，先卸载，再注册 service
	{
		Uninstall_();
		return Install_();
	}
	inline BOOL UnregisterServer_() //卸载服务
	{
		return Uninstall_();
	}

	int Start_() //使用自己启动服务,不要用ATL提供的。
	{
		m_bService = TRUE;
		SERVICE_TABLE_ENTRY st[] = {
			{ m_szServiceName, _ServiceMain },
			{ nullptr, nullptr }
		};
		m_status.dwWin32ExitCode = S_OK;
		if (!::StartServiceCtrlDispatcher(st))
			m_status.dwWin32ExitCode = GetLastError();
		return (int)m_status.dwWin32ExitCode;
	}
};

LPCTSTR FindOneOf(LPCTSTR p1, LPCTSTR p2) //命令行分析
{
	while (p1 && *p1) {
		LPCTSTR p = p2;
		while (p && *p) {
			if (*p1 == *p)
				return CharNext(p1);
			p = CharNext(p);
		}
		p1 = CharNext(p1);
	}
	return nullptr;
}

void ShowMsg()
{
	MessageBox(nullptr, _T("    本程序需要安装为服务程序后台运行,管理员权限命令行窗口安装和卸载如下\n")\
		OPCSERVICENAME\
		_T(" -install\n")\
		OPCSERVICENAME\
		_T(" -uninstall\n\n 详情请见说明书!\n\n 重庆唐码软件有限公司，Copyright(C) 2009-2021"),
		OPCSERVICENAME, MB_OK);
}

CatlopcsrvModule _AtlModule; // ATL的全局实例对象。

//windows规范的main函数入口。
extern "C" int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nShowCmd)
{
	TCHAR szTokens[] = _T("-/");
	LPCTSTR lpszToken = FindOneOf(lpCmdLine, szTokens);

	if (!lpszToken)
		ShowMsg();
	else if (!lstrcmpi(lpszToken, _T("uninstall")) || !lstrcmpi(lpszToken, _T("unreg"))) {
		if (OE_OK != opcsvr_unreg(g_sClsID, g_sVendor, g_sDrivername, g_sVersion, g_sOpcServerDes))
			MessageBox(nullptr, _T("卸载OPC DCOM配置服务失败,请检查是否有管理员权限!"), OPCSERVICENAME, MB_OK);
		_AtlModule.UnregisterServer_();//再卸载服务
	}
	else if (!lstrcmpi(lpszToken, _T("install")) || !lstrcmpi(lpszToken, _T("reg"))) {
		if (!_AtlModule.RegisterServer_()) {//先安装服务
			MessageBox(nullptr, _T("安装服务失败,请检查是否有管理员权限!"), OPCSERVICENAME, MB_OK);
			return 0;
		}
		if (OE_OK != opcsvr_reg(g_sClsID, g_sVendor, g_sDrivername, g_sVersion, g_sOpcServerDes, 1)) {//再安装COM/DCOM组件,注册为service
			_AtlModule.UnregisterServer_();//卸载刚才安装的服务
			MessageBox(nullptr, _T("安装OPC DCOM配置服务失败,请检查是否有管理员权限!"), OPCSERVICENAME, MB_OK);
			return 0;
		}
		MessageBox(nullptr, _T("install opc success!"), OPCSERVICENAME, MB_OK);
	}
	else if (!lstrcmpi(lpszToken, _T("service")) || !lstrcmpi(lpszToken, _T("Embedding")))
		return _AtlModule.Start_();
	else
		ShowMsg();
	return 0;
}