// ntservice.h
#pragma once
#include "tchar.h"
#include "string.h"
#include <stdio.h>
#include <stdarg.h>
#include <Windows.h>
#include "ntserv_msg.h"

class CNtService
{
public:	
	CNtService(){};
	virtual ~CNtService(){};
	void Init( LPCTSTR sName,LPCTSTR sDispName,LPCTSTR sDescription)
	{
		m_bShutDown = FALSE;
		_tcscpy(m_szServiceName,sName);
		_tcscpy(m_szServiceDispName,sDispName);
		_tcscpy(m_szServiceDes,sDescription);
	
		m_hServiceStatus = NULL;
		m_status.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
		m_status.dwCurrentState = SERVICE_STOPPED;
		m_status.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
		m_status.dwWin32ExitCode = 0;
		m_status.dwServiceSpecificExitCode = 0;
		m_status.dwCheckPoint = 0;
		m_status.dwWaitHint = 0;
	}
public:
	static CNtService* _pobj;
	TCHAR m_szServiceName[256];
	TCHAR m_szServiceDispName[256];
	TCHAR m_szServiceDes[256]; 
	
	
    SERVICE_STATUS_HANDLE	m_hServiceStatus;
    SERVICE_STATUS			m_status;
	DWORD					m_dwThreadID;

	bool					m_bShutDown;

	void ServiceMain(DWORD dwArgc, LPTSTR* lpszArgv)
	{
		m_status.dwCurrentState = SERVICE_START_PENDING;
		m_hServiceStatus = RegisterServiceCtrlHandler(m_szServiceName, _Handler);
		if (m_hServiceStatus == NULL)
		{
			LogEvent(CNTS_MSG_SERVICE_FAIL_OPEN_SERVICE,m_szServiceDispName);
			return;
		}
		SetServiceStatus(SERVICE_START_PENDING);
	
		m_status.dwWin32ExitCode = S_OK;
		m_status.dwCheckPoint = 0;
		m_status.dwWaitHint = 0;
	
		Run();
    
		if(m_bShutDown)
			LogEvent(CNTS_MSG_SERVICE_SHUTDOWN,m_szServiceDispName);
		else
			LogEvent(CNTS_MSG_SERVICE_STOPPED,m_szServiceDispName);
	
		SetServiceStatus(SERVICE_STOPPED);
	}
	
    void Handler(DWORD dwOpcode)
	{
		switch (dwOpcode)
		{
		case SERVICE_CONTROL_STOP:
			SetServiceStatus(SERVICE_STOP_PENDING);
			PostThreadMessage(m_dwThreadID, WM_QUIT, 0, 0);
			break;
		case SERVICE_CONTROL_PAUSE:
			break;
		case SERVICE_CONTROL_CONTINUE:
			break;
		case SERVICE_CONTROL_INTERROGATE:
			break;
		case SERVICE_CONTROL_SHUTDOWN:
			m_bShutDown = TRUE;
			SetServiceStatus(SERVICE_STOP_PENDING);
			PostThreadMessage(m_dwThreadID, WM_QUIT, 0, 0);
			break;
		default:
			break;
		}
	}

	void Start()
	{
		SERVICE_TABLE_ENTRY st[] =
		{
			{ m_szServiceName, _ServiceMain },
			{ NULL, NULL }
		};
		if (!::StartServiceCtrlDispatcher(st))
			 Run();
	};
	
	BOOL IsInstalled()
	{
		BOOL bResult = FALSE;
	
		SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	
		if (hSCM != NULL)
		{
			SC_HANDLE hService = ::OpenService(hSCM, m_szServiceName, SERVICE_QUERY_CONFIG);
			if (hService != NULL)
			{
				bResult = TRUE;
				::CloseServiceHandle(hService);
			}
			::CloseServiceHandle(hSCM);
		}
		return bResult;
	}

    BOOL Install()
	{
		if (IsInstalled())
			return TRUE;

		SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
		if (hSCM == NULL)
		{
			MessageBox(NULL, _T("Couldn't open service manager"), m_szServiceName, MB_OK);
			return FALSE;
		}

		TCHAR szFilePath[_MAX_PATH];
		::GetModuleFileName(NULL, szFilePath, _MAX_PATH);

		TCHAR szBinfile[_MAX_PATH];
		_tcscpy(szBinfile,szFilePath);
		_tcscat(szBinfile,_T(" -service"));

		SC_HANDLE hService = ::CreateService(
			hSCM, m_szServiceName, m_szServiceDispName,
			0, SERVICE_WIN32_OWN_PROCESS,
			SERVICE_AUTO_START,  
			SERVICE_ERROR_IGNORE, 
			szBinfile, NULL, NULL, NULL, NULL, NULL);
	
		if (hService == NULL)
		{
			::CloseServiceHandle(hSCM);
			MessageBox(NULL, _T("Couldn't create service"), m_szServiceName, MB_OK);
			return FALSE;
		}

		InstallEventLogSource(m_szServiceName,szFilePath,EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE); 

		TCHAR sKey[_MAX_PATH];
		_tcscpy(sKey,_T("SYSTEM\\CurrentControlSet\\Services\\"));
		_tcscat(sKey,m_szServiceName);
	
		HKEY hkey;
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, sKey, 0, KEY_WRITE|KEY_READ, &hkey) == ERROR_SUCCESS)
		{
			RegSetValueEx(hkey, _T("Description"), NULL, REG_SZ, (LPBYTE)(m_szServiceDes), (lstrlen(m_szServiceDes)+1)*sizeof(TCHAR));
			RegCloseKey(hkey);
		}
		LogEvent(CNTS_MSG_SERVICE_INSTALLED,m_szServiceDispName);

		::CloseServiceHandle(hService);
		::CloseServiceHandle(hSCM);

		MessageBox(NULL, _T("install service success!"), m_szServiceName, MB_OK);
		return TRUE;
	}
    BOOL Uninstall()
	{
		if (!IsInstalled())
			return TRUE;

		SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

		if (hSCM == NULL)
		{
			MessageBox(NULL, _T("Couldn't open service manager"), m_szServiceName, MB_OK);
			return FALSE;
		}

		SC_HANDLE hService = ::OpenService(hSCM, m_szServiceName, SERVICE_STOP | DELETE);

		if (hService == NULL)
		{
			::CloseServiceHandle(hSCM);
			MessageBox(NULL, _T("Couldn't open service"), m_szServiceName, MB_OK);
			return FALSE;
		}
		SERVICE_STATUS status;
		::ControlService(hService, SERVICE_CONTROL_STOP, &status);

		BOOL bDelete = ::DeleteService(hService);
		::CloseServiceHandle(hService);
		::CloseServiceHandle(hSCM);


		UninstallEventLogSource(m_szServiceName); 

		if (bDelete)
		{
			LogEvent(CNTS_MSG_SERVICE_UNINSTALLED,m_szServiceDispName);
			MessageBox(NULL, _T("delete service success!"), m_szServiceName, MB_OK);
			return TRUE;
		}

		MessageBox(NULL, _T("Service could not be deleted"), m_szServiceName, MB_OK);
		return FALSE;
	}
	void LogEvent(UINT ueventid,LPCTSTR pszFormat, ...)
	{
		TCHAR    chMsg[256];
		HANDLE  hEventSource;
		LPTSTR  lpszStrings[1];
		va_list pArg;

		va_start(pArg, pszFormat);
		_vstprintf(chMsg, pszFormat, pArg);
		va_end(pArg);

		lpszStrings[0] = chMsg;
   
		hEventSource = RegisterEventSource(NULL, m_szServiceName);
		if (hEventSource != NULL)
		{			
			ReportEvent(hEventSource, EVENTLOG_INFORMATION_TYPE, 0, ueventid, NULL, 1, 0, (LPCTSTR*) &lpszStrings[0], NULL);
			DeregisterEventSource(hEventSource);
		}
   
	}
    void SetServiceStatus(DWORD dwState)
	{	
		m_status.dwCurrentState = dwState;
		::SetServiceStatus(m_hServiceStatus, &m_status);
	}
	
	BOOL RegisterServer()
	{
		Uninstall();
		return Install();
	}
	inline BOOL UnregisterServer()
	{
		return Uninstall();
	}
		
protected:
	BOOL InstallEventLogSource(LPCTSTR lpSourceName, LPCTSTR lpEventMessageFile, DWORD dwTypesSupported)
	{
		BOOL bSuccess = FALSE;
		HKEY hAppKey;
		if (::RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\"), 0, 
			KEY_WRITE|KEY_READ, &hAppKey) == ERROR_SUCCESS)
		{
			DWORD dw;
			HKEY hSourceKey;
			if (RegCreateKeyEx(hAppKey, lpSourceName, 0, REG_NONE, REG_OPTION_NON_VOLATILE, KEY_WRITE|KEY_READ, NULL,
				&hSourceKey, &dw) == ERROR_SUCCESS)
			{
				bSuccess = (RegSetValueEx(hSourceKey, _T("EventMessageFile"), NULL, REG_SZ, (LPBYTE)lpEventMessageFile, 
					(lstrlen(lpEventMessageFile)+1)*sizeof(TCHAR)) == ERROR_SUCCESS);
			
				bSuccess = bSuccess && (RegSetValueEx(hSourceKey, _T("TypesSupported"), NULL, REG_DWORD,
					(LPBYTE)&dwTypesSupported, sizeof(dwTypesSupported)) == ERROR_SUCCESS);
			
				::RegCloseKey(hSourceKey);
			}
			::RegCloseKey(hAppKey);
		}
		return bSuccess;
	}

	BOOL UninstallEventLogSource(LPCTSTR lpSourceName)
	{
		TCHAR sSubKey[256];
		_tcscpy(sSubKey,_T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\"));
		_tcscat(sSubKey,lpSourceName);
	
		return (RegDeleteKey(HKEY_LOCAL_MACHINE, sSubKey) == ERROR_SUCCESS);
	}

private:
	static void WINAPI _ServiceMain(DWORD dwArgc, LPTSTR* lpszArgv)
	{
		CNtService::_pobj->ServiceMain(dwArgc, lpszArgv);
	};
    static void WINAPI _Handler(DWORD dwOpcode)
	{
		CNtService::_pobj->Handler(dwOpcode); 
	}
protected:	
	virtual void Run()
	{
		m_dwThreadID = GetCurrentThreadId();
		LogEvent(CNTS_MSG_SERVICE_STARTED,m_szServiceDispName);
		SetServiceStatus(SERVICE_RUNNING);
	
		MSG msg;
		while (GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		m_dwThreadID = 0;
	};
public:

	virtual void Debug()
	{
		m_dwThreadID = GetCurrentThreadId();
		MSG msg;
		while (GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		m_dwThreadID = 0;
	};
};
