﻿/*! 
	\file opcsrv_da.h
	\brief OPC Server SDK开发包定义文件  
 
	OPC Server SDK开发包定义文件  
  
	\author 重庆唐码软件有限公司
	\version 3.0
	\date   2010-2015
	*/

#ifndef OPCSRV_DA_H
#define OPCSRV_DA_H

#define	ODT_VOID		0   ///< void
#define	ODT_I8			1	///< char
#define	ODT_U8			2	///< unsigned char
#define	ODT_I16			3	///< short
#define	ODT_U16			4	///< unsigned short
#define	ODT_I32			5	///< int
#define	ODT_U32			6	///< unsigned int
#define	ODT_I64			7	///< __int64
#define	ODT_U64			8	///< unsigned __int64
#define	ODT_F32			9	///< float
#define	ODT_F64			10	///< double
#define ODT_STR			11  ///< char* 或 BSTR
#define ODT_BLOB		12	///< 二进制字节流 (VT_UI1 | VT_ARRAY)
#define ODT_BOOL        13  ///< VT_BOOL类型

#define OTEM_NOWRITE	0	///< 客户端不可写,只读
#define OTEM_WRITE		1	///< 客户端可写

#define OE_OK			0	///< 成功
#define OE_ERR			1	///< 失败
#define OE_ERRARG		2	///< 错误的参数
#define OE_OUTMEM		3	///< 内存不够
#define OE_CLSID		4	///< CLSID错误
#define OE_ERRCOM		5	///< COM/DCOM错误
#define OE_INITCOM		6	///< 初始化COM套间出错

#define OE_NOITEM		10	///< 无此标签
#define OE_ERRTYPE		11	///< 数据类型错误或无法转换数据
#define OE_ERRDATA		12	///< 错误的数据
#define OE_ERRTIME		13	///< 错误的时标
#define OE_ERRQA		14	///< 错误的数据质量
#define OE_NODATA		15	///< 无数据
#define OE_RUNING		16  ///< 已经在运行
#define OE_NOTRUN		17  ///< 没有运行

//{{ OPC 数据质量,从OPC标准复制而来
#define OPC_QUALITY_MASK						0xc0 ///< OPC_QUALITY_MASK
#define OPC_STATUS_MASK							0xfc ///< OPC_STATUS_MASK
#define OPC_LIMIT_MASK							0x03 ///< OPC_LIMIT_MASK
#define OPC_QUALITY_BAD							0x00 ///< 坏质量，常用
#define OPC_QUALITY_UNCERTAIN					0x40 ///< OPC_QUALITY_UNCERTAIN
#define OPC_QUALITY_GOOD						0xc0 ///< 好质量，常用
#define OPC_QUALITY_CONFIG_ERROR				0x04 ///< 配置错误
#define OPC_QUALITY_NOT_CONNECTED				0x08 ///< 设备没有连接
#define OPC_QUALITY_DEVICE_FAILURE				0x0c ///< 设备错误,坏质量细分
#define OPC_QUALITY_SENSOR_FAILURE				0x10 ///< 传感器错误,坏质量细分
#define OPC_QUALITY_LAST_KNOWN					0x14 ///< OPC_QUALITY_LAST_KNOWN
#define OPC_QUALITY_COMM_FAILURE				0x18 ///< OPC_QUALITY_COMM_FAILURE
#define OPC_QUALITY_OUT_OF_SERVICE				0x1c ///< OPC_QUALITY_OUT_OF_SERVICE
#define OPC_QUALITY_WAITING_FOR_INITIAL_DATA	0x20 ///< OPC_QUALITY_WAITING_FOR_INITIAL_DATA
#define OPC_QUALITY_LAST_USABLE					0x44 ///< OPC_QUALITY_LAST_USABLE
#define OPC_QUALITY_SENSOR_CAL					0x50 ///< OPC_QUALITY_SENSOR_CAL
#define OPC_QUALITY_EGU_EXCEEDED				0x54 ///< OPC_QUALITY_EGU_EXCEEDED
#define OPC_QUALITY_SUB_NORMAL					0x58 ///< OPC_QUALITY_SUB_NORMAL
#define OPC_QUALITY_LOCAL_OVERRIDE				0xd8 ///< OPC_QUALITY_LOCAL_OVERRIDE
#define OPC_LIMIT_OK							0x00 ///< OPC_LIMIT_OK
#define OPC_LIMIT_LOW							0x01 ///< OPC_LIMIT_LOW
#define OPC_LIMIT_HIGH							0x02 ///< OPC_LIMIT_HIGH
#define OPC_LIMIT_CONST							0x03 ///< OPC_LIMIT_CONST
//}}

///\brief 客户端写标签值回调函数，以便写入到具体的设备中。
typedef bool (__stdcall *cb_SetItemVal)(const char* sTagName,VARIANT* pval,unsigned short wqa,long long filetime,void* pCallBackArg);

#ifdef __cplusplus
extern "C" {
#endif
	/*!
	\brief 注册OPC Server
	\param sClsID [in]CLSID 比如"{DEAFC4D7-1B56-4eff-B7DC-D06999F47ADB}",ANSI字符集,<80字符
	\param sVendor [in] 提供商,比如"tom",ANSI字符集,<16字符
	\param sDrivername [in] 驱动名,比如"rdb4opcda",ANSI字符集,<16字符
	\param sVersion [in] 版本名,比如"1" ,ANSI字符集,<16字符
	\param sOpcServerDes [in] OPC Server描述 ，ANSI字符集,<128字符	
	\param AsService [in] 0前台程序，非0后台service	
	\return OE_OK表示成功，否则返回错误码.
	*/
	int __stdcall opcsvr_reg(const char* sClsID,const char* sVendor,const char* sDrivername,const char* sVersion,const char* sOpcServerDes,int AsService);

	/*!
	\brief 卸载OPC Server
	\param sClsID [in]CLSID 比如"{DEAFC4D7-1B56-4eff-B7DC-D06999F47ADB}",ANSI字符集,<80字符
	\param sVendor [in] 提供商,比如"tom",ANSI字符集,<80字符
	\param sDrivername [in] 驱动名,比如"rdb4opcda",ANSI字符集,<80字符
	\param sVersion [in] 版本名,比如"1" ,ANSI字符集,<80字符
	\param sOpcServerDes [in] OPC Server描述 ，ANSI字符集,<128字符	
	\return OE_OK表示成功，否则返回错误码.
	*/
	int __stdcall opcsvr_unreg(	const char* sClsID,	const char* sVendor,const char* sDrivername,const char* sVersion,const char* sOpcServerDes);
	
	/*!
	\brief 添加标签
	\param sName [in]标签名, ANSI字符集, <80字符
	\param sDes [in] 标签表述 ,ANSI字符集,< 80字符
	\param sUnit [in] 标签单位, ANSI字符集,< 16字符
	\param nType [in] 数据类型 ODT_I32等 参见 \ref sec_datatype
	\param nWriteable [in] 可写标志 OTEM_NOWRITE 或者OTEM_WRITE	参见 \ref sec_rw
	\return OE_OK表示成功，否则返回错误码.
	\remark 可以任何时候添加标签,最好在opcsrv_start之前全部添加完标签,保证客户端连接后可以立即读取可用标签定义或者立即添加标签组。
	*/
	int	__stdcall opcsrv_addtag(const char* sName,	const char* sDes,const char*	sUnit,int nType, int nWriteable);

	/*!
	\brief 启动OPC Server
	\param sClsID [in]CLSID 比如"{DEAFC4D7-1B56-4eff-B7DC-D06999F47ADB}",ANSI字符集,<80字符
	\param sVendor [in] 提供商,比如"tom",ANSI字符集,<80字符
	\param sDrivername [in] 驱动名,比如"rdb4opcda",ANSI字符集,<80字符
	\param sVersion [in] 版本名,比如"1" ,ANSI字符集,<80字符
	\param sOpcServerDes [in] OPC Server描述 ，ANSI字符集,<128字符	
	\param pfcbSetTagVal [in] 回调函数,NULL表示不响应客户端写操作
	\param pcbSetTagValArg [in] 回调参数，pfcbSetTagVal使用的参数
	\param LockServer [in] 非0锁定服务,0不锁定。不锁定时，当所有的客户端断开后，非锁定会退出应用程序，锁定则会继续运行。
	\return OE_OK表示成功，否则返回错误码.
	*/
	int __stdcall opcsrv_start(const char* sClsID,const char* sVendor,const char* sDrivername,const char* sVersion,const char*	sOpcServerDes,cb_SetItemVal pfcbSetTagVal,void*	pcbSetTagValArg,int	LockServer);
	
	/*!
	\brief停止OPC Server
	\return OE_OK表示成功，否则返回错误码.
	\remark 当服务程序退出时才调用这个接口,OPC Server做退出的内存清理工作。
	*/
	int __stdcall opcsrv_stop();

	/*!
	\brief写标签值,全部类型,ODT_I8到ODT_BLOB全部支持
	\param  sTagname [in] 标签名,ANSI字符集,<80字符
	\param  pval [in] 值,C# 中为ref object,参见tstopc_cs工程中的例子
	\param  wqa [in] 质量,参见 \ref sec_opcqa
	\param  filetime [in] 时标，filetime时标,是用一个64未得整数.MSDN原文描述：A file time is a 64-bit value that represents the number 
	of 100-nanosecond intervals that have elapsed since 12:00 A.M. January 1, 1601 Coordinated Universal Time (UTC). 
	*/
	int	__stdcall opcsrv_setval(const char* sTagname,VARIANT* pval,	unsigned short wqa,	__int64  filetime);
	
	//下面两个不是必须的，只是方便C/C++的使用，完全可以使用opcsrv_setval替代。

	/*!
	\brief写字符串标签值,不是必须的,只是方便C/C++使用,可以用opcsrv_setval替换,参见C/C++例程tstopc_cp
	\brief写标签值,全部类型,ODT_I8到ODT_BLOB全部支持
	\param  sTagname [in] 标签名,ANSI字符集,<80字符
	\param  val [in] ASNI字符串值,0字符结束
	\param  wqa [in] 质量,参见 \ref sec_opcqa
	\param  filetime [in] 时标，filetime时标,是用一个64未得整数.MSDN原文描述：A file time is a 64-bit value that represents the number 
	of 100-nanosecond intervals that have elapsed since 12:00 A.M. January 1, 1601 Coordinated Universal Time (UTC). 
	*/
	int	__stdcall opcsrv_setstr(const char* sTagname,const char* val,unsigned short wqa,__int64  filetime);

	/*!
	\brief写二进制流标签值,不是必须的,只是方便C/C++使用,可以用opcsrv_setval替换,参见C/C++例程tstopc_cp
	\brief写标签值,全部类型,ODT_I8到ODT_BLOB全部支持
	\param  sTagname [in] 标签名,ANSI字符集,<80字符
	\param  val [in] 字节数组,0字符结束
	\param  nsize [in] val的字节数
	\param  wqa [in] 质量,参见 \ref sec_opcqa
	\param  filetime [in] 时标，filetime时标,是用一个64未得整数.MSDN原文描述：A file time is a 64-bit value that represents the number 
	of 100-nanosecond intervals that have elapsed since 12:00 A.M. January 1, 1601 Coordinated Universal Time (UTC). 
	*/
	int	__stdcall opcsrv_setblob(const char* sTagname,unsigned char val[],int nsize,unsigned short wqa,	__int64  filetime);

#ifdef __cplusplus
}
#endif

#endif //OPCSRV_DA_H

/*! \mainpage 首页
OPC Server DA开发包,适合C/C++,C#,VB等常用开发工具使用,附带有 C/C++ 和 C# 的例子。<br>
本开发包屏蔽了所有的OPC和COM/DCOM的内容,只需简单几个函数即可实现一个OPC Server服务器。 <br>
\image html flow.png <br>
四个函数调用就完成一个OPC Server的服务。 <br><br>
因为OPC Server是COM/DCOM组件,因此在系统里组要调用 \ref opcsvr_reg 注册安装OPC COM/DCOM组件,本质就是一系列的注册表写操作，只
需安装一次即可,可以做到安装包里调用执行。这个接口需要管理员权限,附带的例子在命令行安装，参见tstopc_cp和tstopc_cs例程。<br>
如果需要从系统卸载OPC Server，只需调用 \ref opcsvr_unreg 即可,本质也是把 \ref opcsvr_reg 写入的注册表信息清理干净。
<br><br>
\ref pg_def <br>
\ref pg_fun <br>
\ref pg_opcver  <br>
\ref pg_opcdemo <br> <br>
友情提示：
我公司网上发布的为DEMO版,有时间限制，每次启动8小时候停止数据服务，其他功能无限制。如需要正版，请按照如下联系方式联系。
<hr>
重庆唐码软件有限公司 <br>
QQ:459222361 <br>
email:13212314895@126.com <br>
TEL:023-65467771,13212314895 <br>
网站: www.tomchina.cn <br>
联系人:蒋工 <br>

\page pg_def 常数定义
	\section sec_datatype 数据类型
	类型          | 值    |意义
    --------------|-------|------
	\ref ODT_VOID | 0|\copybrief ODT_VOID <br>
	\ref ODT_I8   | 1|\copybrief ODT_I8 <br>
	\ref ODT_U8   | 2|\copybrief ODT_U8 <br>
	\ref ODT_I16  | 3|\copybrief ODT_I16 <br>
	\ref ODT_U16  | 4|\copybrief ODT_U16 <br>
	\ref ODT_I32  | 5|\copybrief ODT_I32 <br>
	\ref ODT_U32  | 6|\copybrief ODT_U32 <br>
	\ref ODT_I64  | 7|\copybrief ODT_I64 <br>
	\ref ODT_U64  | 8|\copybrief ODT_U64 <br>
	\ref ODT_F32  | 9|\copybrief ODT_F32 <br>
	\ref ODT_F64  |10|\copybrief ODT_F64 <br>
	\ref ODT_STR  |11|\copybrief ODT_STR <br>
	\ref ODT_BLOB |12|\copybrief ODT_BLOB <br>
	\ref ODT_BOOL |13|\copybrief ODT_BOOL <br>

	<br>
	\section sec_rw 客户端读写属性
	属性          | 值    |意义
    --------------|-------|------
	\ref OTEM_NOWRITE | 0|\copybrief OTEM_NOWRITE <br>
	\ref OTEM_WRITE   | 1|\copybrief OTEM_WRITE <br>

	<br>
	\section sec_errcode 接口返回错误码
	错误码         | 值    |意义
    --------------|-------|------
	\ref OE_OK | 0|\copybrief OE_OK <br>
	\ref OE_ERR   | 1|\copybrief OE_ERR <br>
	\ref OE_ERRARG   | 2|\copybrief OE_ERRARG <br>
	\ref OE_OUTMEM   | 3|\copybrief OE_OUTMEM <br>
	\ref OE_CLSID   | 4|\copybrief OE_CLSID <br>
	\ref OE_ERRCOM   | 5|\copybrief OE_ERRCOM <br>
	\ref OE_INITCOM   | 6|\copybrief OE_INITCOM <br>
	\ref OE_NOITEM   | 10|\copybrief OE_NOITEM <br>
	\ref OE_ERRTYPE   | 11|\copybrief OE_ERRTYPE <br>
	\ref OE_ERRDATA   | 12|\copybrief OE_ERRDATA <br>
	\ref OE_ERRTIME   | 13|\copybrief OE_ERRTIME <br>
	\ref OE_ERRQA   | 14|\copybrief OE_ERRQA <br>
	\ref OE_NODATA   | 15|\copybrief OE_NODATA <br>
	\ref OE_RUNING   | 16|\copybrief OE_RUNING <br>
	\ref OE_NOTRUN   | 17|\copybrief OE_NOTRUN <br>

	<br>
	\section sec_opcqa OPC数据质量
	质量         | 值    |意义
    --------------|-------|------
	\ref OPC_QUALITY_MASK | 0xc0|\copybrief OPC_QUALITY_MASK <br>
	\ref OPC_STATUS_MASK   | 0xfc|\copybrief OPC_STATUS_MASK <br>
	\ref OPC_LIMIT_MASK   | 0x03|\copybrief OPC_LIMIT_MASK <br>
	\ref OPC_QUALITY_BAD   | 0x00|\copybrief OPC_QUALITY_BAD <br>
	\ref OPC_QUALITY_UNCERTAIN   | 0x40|\copybrief OPC_QUALITY_UNCERTAIN <br>
	\ref OPC_QUALITY_GOOD   | 0xc0|\copybrief OPC_QUALITY_GOOD <br>
	\ref OPC_QUALITY_CONFIG_ERROR   | 0x04|\copybrief OPC_QUALITY_CONFIG_ERROR <br>
	\ref OPC_QUALITY_NOT_CONNECTED   | 0x08|\copybrief OPC_QUALITY_NOT_CONNECTED <br>
	\ref OPC_QUALITY_DEVICE_FAILURE   | 0x0c|\copybrief OPC_QUALITY_DEVICE_FAILURE <br>
	\ref OPC_QUALITY_SENSOR_FAILURE   | 0x10|\copybrief OPC_QUALITY_SENSOR_FAILURE <br>
	\ref OPC_QUALITY_LAST_KNOWN   | 0x14|\copybrief OPC_QUALITY_LAST_KNOWN <br>
	\ref OPC_QUALITY_COMM_FAILURE   | 0x18|\copybrief OPC_QUALITY_COMM_FAILURE <br>
	\ref OPC_QUALITY_OUT_OF_SERVICE   | 0x1c|\copybrief OPC_QUALITY_OUT_OF_SERVICE <br>
	\ref OPC_QUALITY_WAITING_FOR_INITIAL_DATA   | 0x20|\copybrief OPC_QUALITY_WAITING_FOR_INITIAL_DATA <br>
	\ref OPC_QUALITY_LAST_USABLE   | 0x44|\copybrief OPC_QUALITY_LAST_USABLE <br>
	\ref OPC_QUALITY_SENSOR_CAL | 0x50|\copybrief OPC_QUALITY_SENSOR_CAL <br>
	\ref OPC_QUALITY_EGU_EXCEEDED | 0x54|\copybrief OPC_QUALITY_EGU_EXCEEDED <br>
	\ref OPC_QUALITY_SUB_NORMAL |	0x58|\copybrief OPC_QUALITY_SUB_NORMAL <br>
	\ref OPC_QUALITY_LOCAL_OVERRIDE | 0xd8|\copybrief OPC_QUALITY_LOCAL_OVERRIDE <br>
	\ref OPC_LIMIT_OK | 0x00|\copybrief OPC_LIMIT_OK <br>
	\ref OPC_LIMIT_LOW	| 0x01|\copybrief OPC_LIMIT_LOW <br>
	\ref OPC_LIMIT_HIGH |	0x02|\copybrief OPC_LIMIT_HIGH <br>
	\ref OPC_LIMIT_CONST |	0x03|\copybrief OPC_LIMIT_CONST <br>

\page pg_fun 接口函数
接口函数很简单，安装注册后，最少只要5个即可工作。
<br>
	\section sec_fun 接口函数一览表
	函数 | 简述值
  -------|--------
  \ref opcsvr_reg   | \copybrief opcsvr_reg <br>
  \ref opcsvr_unreg  | \copybrief opcsvr_unreg <br>
  \ref opcsrv_addtag  | \copybrief opcsrv_addtag <br>
  \ref opcsrv_start   | \copybrief opcsrv_start <br>
  \ref opcsrv_stop   | \copybrief opcsrv_stop <br>
  \ref opcsrv_setval | \copybrief opcsrv_setval <br>
  \ref opcsrv_setstr | \copybrief opcsrv_setstr <br>
  \ref opcsrv_setblob | \copybrief opcsrv_setblob <br>

\page pg_opcver 本开发包支持的OPC的接口
	本开发包支持OPC Server DA1.0 同步和异步，DA2.0同步和异步，DA3.0异步。具体支持如下接口： <br>
1）Server层接口 <br>
IUnknown		// COM标准Unknown接口 <br>
IOPCServer		// Server接口 <br>
IOPCCommon 	// 公共接口，主要是本地语言设置 <br>
IOPCItemIO		// DA 3.0简易读写接口 <br>
IOPCItemProperties // 标签属性接口 <br>
IOPCBrowseServerAddressSpace // 标签检索接口 <br>
 <br>
2）Group层接口 <br>
IUnknown			// COM标准Unknown接口 <br>
IOPCItemMgt			// item管理接口 <br>
IOPCSyncIO			// DA1.0同步 <br>
IDataObject     	// DA1.0异步回调
IOPCAsyncIO         // DA1.0异步 <br>
IOPCSyncIO2			// DA2.0同步 <br>
IOPCAsyncIO2		// DA2.0异步 <br>
IOPCAsyncIO3		// DA3.0异步 <br>
IOPCGroupStateMgt  	//组状态管理接口 <br>
IConnectionPointContainer //连接点容器接口，含有IOPCDataCallback通知 <br>
 <br>
	从上面可看出，在DA1.0,2.0,3.0中全部支持。 <br>

\page pg_opcdemo C/C++,C# 例子程序安装及测试
为方便理解开发包的使用,提供了以下C/C++和C# 的例子工程。 <br>

例子 | 简述
  -------|--------
  tstopc_cp   | 控制台C/C++例程 <br>
  tstopc_cs   | 控制台C# 例程 <br>
  tstopc_csw  | 窗口版C# 例程 <br>
  tstopc_mfc  | MFC版C/C++ 例程 <br>
  tstopc_w32  | win32 win api窗口版C/C++ 例程 <br>
  tstopc_srv  | C++后台服务版
  tstopc_vbwin| VB.net版本

  <br>
第一步:安装OPC基金会的底层发布包 <br>
OPC Core Components Redistributable (x86) 105.1.zip <br> <br>
第二步:将opcsrv_da.dll,tstopc_cp.exe和tstopc_cs.exe拷贝到一个目录，比如d:/myopc <br>
\image html files.png <br>
然后安装OPC Server即可,注意需要管理员权限。 <br>
\image html reg.png <br>

安装完成后即可用第三方客户端工具测试效果了。本开发包附带一个KEPware的免费客户端工具opctestclient.exe可供参考。
<br>
*/